/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <wipal/phy/avs_header.hh>

#include "wscout_gui_avs_header.hh"

namespace wscout
{

  namespace gui
  {

    AVSHeader::next_header
    AVSHeader::setup(const void*			h,
		       size_t				caplen,
		       const Trace&			,
		       const pcapxx::frame_descriptor&	)
    {
      const avs::header* const	avs = static_cast<const avs::header*> (h);

      if (caplen < avs->len(caplen, false))
	{
	  setTruncated();
	  return next_header (protocol::NONE, 0);
	}

      setUpdatesEnabled(false);
      setTruncated(false);
      setTime(avs->hosttime_get(), avs->mactime_get());
      setRate(tool::extract_big_endian_long_u(avs->datarate) / 10.f);
      setChannel(tool::extract_big_endian_long_u(avs->channel));
      setSignal(-1,
		tool::extract_big_endian_long_u(avs->ssi_signal),
		tool::extract_big_endian_long_u(avs->ssi_noise));
      setUpdatesEnabled(true);

      return next_header (protocol::IEEE802_11,
			  avs->decapsulate(caplen, false));
    }

  } // End of namespace gui.

} // End of namespace wscout.
