/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <QtGui/QToolTip>
#include <QtGui/QHelpEvent>

#include <boost/foreach.hpp>

#include "wscout_gui_packet.hh"
#include "wscout_gui_abstract_header.hh"
#include "wscout_hexdump.hh"

namespace wscout
{

  namespace gui
  {

    void
    Packet::setup(const Trace& t, const packet& pkt)
    {
      setUpdatesEnabled(false);
      setToolTip(hexdump(pkt.bytes().get(),
			 pkt.pcap_header()->caplen).c_str());

      BOOST_FOREACH(AbstractHeader* h, headers_)
	delete h;
      headers_.clear();

      AbstractHeader::next_header next (protocol::PCAP, pkt.bytes().get());

      while (true)
	{
	  const uint8_t* const	b = static_cast<const uint8_t*> (next.second);
	  const uint8_t* const	e = pkt.bytes().get();
	  const size_t		caplen = pkt.pcap_header()->caplen - (b - e);

	  protocol_map&			m = protocols();
	  protocol_map::iterator	p = m.find(next.first);

	  if (p == m.end())
	    break;

	  AbstractHeader* const	h = p->second();
	  headers_.push_back(h);
	  next = h->setup(next.second, caplen, t, pkt);

	  HeaderFrame* const	f = dynamic_cast<HeaderFrame*> (h);
	  assert(f);
	  layout()->addWidget(f);
	  f->show();
	}

      updateMinimumSize();
      setUpdatesEnabled(true);
    }

  } // End of namespace gui.

} // End of namespace wscout.
