/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WSCOUT_GUI_PIXMAP_HH_
# define WSCOUT_GUI_PIXMAP_HH_

# include <QtGui/QPixmap>

namespace wscout
{

  namespace gui
  {

    /// Hold the Qt prefix for the pixmap search path.
    extern const QString	pixmap_search_path_prefix;

    /**
     * Initialize the pixmap search paths.
     *
     * The pixmap search paths are composed of:
     *
     * <ol>
     *
     *   <li>The directories listed in the @c PIXMAP_PATH environment variable,
     *       separated by semi-columns,</li>
     *
     *   <li> @code $(top_builddir)/pixmaps @endcode </li>
     *
     *   <li> @code $(top_srcdir)/pixmaps @endcode </li>
     *
     *   <li> @code $(pixmapdir) @endcode
     *        (@c /usr/local/share/wscout/pixmaps by default) </li>
     *
     *   <li>The application's internal resources, in the @c /pixmaps
     *       directory.</li>
     *
     * </ol>
     *
     * Pixmaps are searched in those directories in ascending order.
     */
    void
    pixmapInit();

    /**
     * Search and retrieve a pixmap inside the pixmap search paths.
     *
     * @e e.g. when given @c "foo" this function might search for
     * @c /some/path/foo.svg, @c /some/other/path/foo.png, etc.
     */
    QPixmap
    pixmap(const QString& name);

  } // End of namespace gui.

} // End of namespace wscout.

# include "wscout_gui_pixmap.hxx"

#endif // ! WSCOUT_GUI_PIXMAP_HH_
