/*
 * WScout - Lightweight PCAP visualizer.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#ifndef WSCOUT_PROTOCOLS_IDS_HH_
# define WSCOUT_PROTOCOLS_IDS_HH_

namespace wscout
{

  /// Hold %protocol identifiers.
  namespace protocol
  {

    /**
     * Identify a %protocol.
     *
     * Usually identify the content of a pointer to some data.
     *
     * @see gui::AbstractHeader::next_header
     */
    enum id
      {
	NONE,		///< Data does not refer to a specific %protocol.
	UNKNOWN,	///< Data refers to an unkonwn %protocol.
	ENCRYPTED,	///< Data is an encrypted payload.

	PCAP,		///< Data refers to a PCAP header.
	PRISM_HEADER,	///< Data is a Prism header. @see gui::PrismHeader.
	AVS_HEADER,	///< Data is an AVS header. @see gui::AVSHeader.

	IEEE802_11,		///< Data is a IEEE 802.11 frame.
				///  @see gui::IEEE80211Header.
	IEEE802_11_BEACON,	///< Data is the body of a 802.11 beacon.
	IEEE802_11_DISASSOC,	///< Data is the body of a 802.11 disassoc.
	IEEE802_11_ASS_REQ,	///< Data is the body of a 802.11 ass. req.
	IEEE802_11_ASS_RESP,	///< Data is the body of a 802.11 ass. resp.
	IEEE802_11_REASS_REQ,	///< Data is the body of a 802.11 reass. req.
	IEEE802_11_REASS_RESP,	///< Data is the body of a 802.11 reass. resp.
	IEEE802_11_PROBE_RESP	///< Data is the body of a 802.11 probe resp.

			// FIXME: The following is an ugly hack: probe
			// FIXME: response frame bodies do not have the same
			// FIXME: format as beacon frame bodies.  Yet the
			// FIXME: difference is small and not relevant to
			// FIXME: our simple parser.
			= IEEE802_11_BEACON,

	IEEE802_11_PROBE_REQ,	///< Data is the body of a 802.11 probe req.
	IEEE802_11_AUTH,	///< Data is the body of a 802.11 auth.
	IEEE802_11_DEAUTH,	///< Data is the body of a 802.11 deauth.
	IEEE802_11_ACTION,	///< Data is the body of a 802.11 action frame.

	LLC,		///< Data is a Logical Link Control frame.
	IPv4,		///< Data is an IPv4 datagram.
	IPv6,		///< Data is an IPv6 datagram.
	ARP		///< Data is an ARP packet.
      };

  } // End of namespacce protocol.

} // End of namespace wscout.

#endif // ! WSCOUT_PROTOCOLS_IDS_HH_
