/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */

#include <iostream>
#include <stdexcept>
#include <boost/foreach.hpp>

#include "config.hh"

#include <wipal/wifi/frame/stats.hh>
#include <wipal/tool/options.hh>
#include <wipal/pcap/list_of_traces.hh>
#include <wipal/wifi/frame/filter/microseconds_stamper.hh>
#include <wipal/wifi/frame/filter/non_noisy_prism.hh>

namespace
{

  template <class PhyHeader>
  struct stats_updater
  {
    stats_updater(wifi::frame::stats::stats& stats): stats_ (stats)
    {
    }

    template <class T>
    void
    operator () (const T& pkt)
    {
      stats_.account<PhyHeader>(pkt);
    }

  private:
    wifi::frame::stats::stats&	stats_;
  };

} // End of anonymous namespace.

int
main(int argc, char *const argv[])
{
  try
    {
      const tool::opt::options	options =
	tool::opt::parse_args(argc, argv,
			      WP_OPT_ENDIANNESS
			      WP_OPT_PRISM_NOISE,
			      tool::opt::options::arbitrary_number_of_inputs,
			      0);

      wifi::frame::stats::stats	stats;

      BOOST_FOREACH(pcapxx::list_of_traces<> t, options.input)
	{
	  stats.clear_states();
	  if (options.prism_noise_means_error)
	    t.expect(pcapxx::descriptor<>::PRISM_HEADER);

	  switch (const pcapxx::descriptor<>::link_type lt =
		    t.iterable(0).linktype())
	    {

#define WP_case(R, Unused, LT)						\
	    case pcapxx::descriptor<>::BOOST_PP_TUPLE_ELEM(2, 0, LT):	\
	      {								\
		typedef	wifi::frame::filter::microseconds_stamper<	\
			  pcapxx::list_of_traces<>::const_iterator,	\
			  BOOST_PP_TUPLE_ELEM(2, 1, LT)>		\
		  ms_stamper_t;						\
									\
		stats_updater<BOOST_PP_TUPLE_ELEM(2, 1, LT)>  update (stats); \
		ms_stamper_t	mss (t.begin(), t.end(), options.endianness); \
									\
		if (options.prism_noise_means_error)			\
		  {							\
		    typedef ms_stamper_t::const_iterator		it_t; \
		    typedef wifi::frame::filter::non_noisy_prism<it_t>	nnp_t; \
									\
		    nnp_t	n (mss.begin(), mss.end());		\
									\
		    n.for_each(update);					\
		  }							\
		else							\
		  mss.for_each(update);					\
									\
	      }								\
	    break;

	      BOOST_PP_SEQ_FOR_EACH(WP_case, ~, WP_LINKTYPES)

	    default:
	      tool::opt::unsupported_linktype(lt);
	      break;
	    }
	}

      std::cout << stats;
    }
  catch (std::exception& e)
    {
      std::cerr << "ERROR: " << e.what() << '.' << std::endl;
      return EXIT_FAILURE;
    }
}
